#include "gmock/gmock.h"
#include <fstream>
#include <memory>

#include "../Logger.h"


TEST(LoggerTest, logStringWithoutSettingLevels_WillPutTheLogMesssageIntoTheStream)
{
    std::ostringstream stream;
    logging::Logger::setStream(&stream);

    *logging::out << logging::Logger::LOW << "Hello World\n";

    EXPECT_THAT(stream.str(), "[LOW] Hello World\n");
}

TEST(LoggerTest, logStringWithHighDebugLevel_logOnlyHighLevelMessages)
{
    std::ostringstream stream;
    logging::Logger::setStream(&stream);

    logging::Logger::setDebugLevel(logging::Logger::HIGH);
    *logging::out << logging::Logger::LOW << "Low Debug Message\n" << logging::Logger::HIGH << "HIGH Debug Message\n";

    EXPECT_THAT(stream.str(), "[HIGH] HIGH Debug Message\n");
}

TEST(LoggerTest, addTwoStreams_shouldWriteToBoth)
{
    logging::Logger::resetStreams();

    std::ostringstream stream1, stream2;
    logging::out->addStream(&stream1);
    logging::out->addStream(&stream2);
    logging::Logger::setDebugLevel(logging::Logger::LOW);

    *logging::out << logging::Logger::LOW <<"Hello World\n";

    EXPECT_THAT(stream1.str(), "[LOW] Hello World\n");
    EXPECT_THAT(stream2.str(), "[LOW] Hello World\n");
}

TEST(LoggerTest, splittetOutputShouldHaveDebugInformationOnce)
{
    std::ostringstream stream;
    logging::Logger::setStream(&stream);

    *logging::out << logging::Logger::LOW << "Hello" << " World\n";

    EXPECT_THAT(stream.str(), "[LOW] Hello World\n");
}

TEST(LoggerTest, enableTimeStampInOutput)
{
    std::ostringstream stream;
    logging::Logger::setStream(&stream);
    logging::Logger::timeStamp(logging::Logger::TimeStamp::ENABLE);

    *logging::out << logging::Logger::LOW << "Hello" << " World\n";
    
    EXPECT_THAT(stream.str(), testing::StrNe("[LOW] Hello World\n"));
}

